'/* BTRIEVE.BAS  27.10.2000 */

dim ibtrvds(14)                                   '/* Btrieve data structure */
ibtrvid = 0                                           '/* Btrieve process ID */
ibtrvmu = 0                          '/* TRUE when multi-user Btrieve loaded */
ibtrvvs = 0                       '/* TRUE when Btrieve version has been set */
ibtrvas = varseg(ibtrvds(0))               '/* current array-segment address */
ibtrvns = varseg(ibtrvid)                '/* current numeric-segment address */
ibtrvss = varseg(char(0))                 '/* current string-segment address */

function BTRV(ioper, cfcb, cbuf, ilen, ckey, ikey)
   istat = 0                                   '/* status returned by BTRV() */

   if not ibtrvvs then                  '/* Btrieve version has not been set */
      ibtrvvs = not 0                       '/* set Btrieve-version-set flag */
      srg.ax = &h3000                                   '/* DOS version code */
      call interruptx(&h21, srg, srg)              '/* check the DOS version */
      if (srg.ax and &h00ff) >= 3 then            '/* DOS version is >= 3.xx */
         srg.ax = BTRV.IV2                          '/* set DOS interrupt 2F */
         call interruptx(BTRV.IVM, srg, srg)       '/* call DOS interrupt 2F */
         ibtrvmu = ((srg.ax and &hff) = asc("M"))  '/* TRUE if multi-user ON */
      end if
      if not ibtrvmu then                  '/* multi-user Btrieve not loaded */
         srg.ax = &h3500 + BTRV.IVS        '/* set test for standard version */
         call interruptx(&h21, srg, srg)    '/* do test for standard version */
         if srg.bx <> BTRV.OWS then    '/* if BX != offset-within-segment... */
            BTRV = BTRV.ERR             '/* Btrieve not loaded; return error */
            exit function                 '/* exit to return the error value */
         end if
      end if
   end if
   ibtrvds(0) = sadd(cbuf)            '/* calling program data buffer offset */
   ibtrvds(1) = ibtrvss              '/* calling program data buffer segment */
   ibtrvds(2) = ilen                  '/* calling program data buffer length */
   ibtrvds(3) = sadd(cfcb) + &h26                        '/* disk FCB offset */
   ibtrvds(4) = ibtrvss                                 '/* disk FCB segment */
   ibtrvds(5) = sadd(cfcb)         '/* calling program position block offset */
   ibtrvds(6) = ibtrvss           '/* calling program position block segment */
   ibtrvds(7) = ioper                             '/* Btrieve operation code */
   ibtrvds(8) = sadd(ckey)             '/* calling program key buffer offset */
   ibtrvds(9) = ibtrvss               '/* calling program key buffer segment */
   ibtrvds(10) = ikey * 256 + 255        '/* key path# and key buffer length */
   ibtrvds(11) = varptr(istat)                      '/* return-status offset */
   ibtrvds(12) = ibtrvns                           '/* return-status segment */
   ibtrvds(13) = BTRV.VLN                      '/* variable-length-record ID */
   srg.dx = varptr(ibtrvds(0))              '/* put parameter offset into DX */
   srg.ds = ibtrvas                        '/* put parameter segment into DS */
   if ibtrvmu then                          '/* multi-user Btrieve is loaded */
      do while ibtrvmu                     '/* begin Btrieve processing loop */
         srg.bx = ibtrvid                         '/* put process ID into BX */
         if srg.bx then                               '/* process ID is TRUE */
            srg.ax = BTRV.IV2 + 2    '/* set AX to int 2F; adjust for procid */
         else                                        '/* process ID is FALSE */
            srg.ax = BTRV.IV2 + 1        '/* set AX to int 2F; no adjustment */
         end if
         call interruptx(BTRV.IVM, srg, srg)     '/* call Btrieve multi-user */
         if (srg.ax and &h00ff) = 0 then    '/* if Btrieve call processed... */
            exit do                         '/* exit Btrieve processing loop */
         end if
      loop
      if ibtrvid = 0 then                    '/* process ID has not been set */
         ibtrvid = srg.bx                            '/* set process ID flag */
      end if
   else                               '/* Btrieve standard version is loaded */
      call interruptx(BTRV.IVS, srg, srg)  '/* call Btrieve standard version */
   end if
   ilen = ibtrvds(2)                    '/* reset data length as appropriate */
   BTRV = istat                 '/* return Btrieve status to calling program */
end function

function io.btrv(ioper, cfcb, cbuf, ilen, ckey, ikey)
   istat = BTRV(ioper, cfcb, cbuf, ilen, ckey, ikey)'/* call Btrieve manager */

   if istat then                         '/* status <> 0 if operation failed */
      select case ioper                                   '/* operation code */
         case BTRV.DELETE, _                     '/* delete a Btrieve record */
              BTRV.UPDATE                        '/* update a Btrieve record */
            if istat = BTRV.CONFLICT then               '/* network conflict */
               istat = 2                          '/* status = 2 if conflict */
            else                                  '/* not a network conflict */
               if ioper = BTRV.UPDATE and istat = BTRV.DUPLICATE then
                  istat = 3                      '/* status = 3 if duplicate */
               else                                      '/* not a duplicate */
                  istat = 1               '/* status = 1 if operation failed */
               end if
            end if
         case BTRV.GETEQ, _                  '/* get an exact Btrieve record */
              BTRV.GETEQKEY                     '/* get an exact Btrieve key */
            if istat = BTRV.NOTFND then                 '/* record not found */
               istat = 2                         '/* status = 2 if not found */
            else                                  '/* not a record-not-found */
               istat = 1                  '/* status = 1 if operation failed */
            end if
         case BTRV.GETGE, _        '/* get a greater-or-equal Btrieve record */
              BTRV.GETGEKEY, _        '/* get a greater-or-equal Btrieve key */
              BTRV.GETGT, _            '/* get a greater-than Btrieve record */
              BTRV.GETGTKEY, _            '/* get a greater-than Btrieve key */
              BTRV.GETHI, _               '/* get the highest Btrieve record */
              BTRV.GETHIKEY, _               '/* get the highest Btrieve key */
              BTRV.GETLE, _      '/* get a less-than-or-equal Btrieve record */
              BTRV.GETLEKEY, _      '/* get a less-than-or-equal Btrieve key */
              BTRV.GETLO, _                '/* get the lowest Btrieve record */
              BTRV.GETLOKEY, _                '/* get the lowest Btrieve key */
              BTRV.GETLT, _               '/* get a less-than Btrieve record */
              BTRV.GETLTKEY, _               '/* get a less-than Btrieve key */
              BTRV.GETNEXT, _                '/* get the next Btrieve record */
              BTRV.GETNEXTKEY, _                '/* get the next Btrieve key */
              BTRV.GETPREV, _            '/* get the previous Btrieve record */
              BTRV.GETPREVKEY               '/* get the previous Btrieve key */
            if istat = BTRV.EOF then                         '/* Btrieve EOF */
               istat = 2                               '/* status = 2 if EOF */
            else                                       '/* not a Btrieve EOF */
               istat = 1                  '/* status = 1 if operation failed */
            end if
         case BTRV.INSERT                        '/* insert a Btrieve record */
            if istat = BTRV.DUPLICATE then              '/* duplicate record */
               istat = 2                         '/* status = 2 if duplicate */
            else                                         '/* not a duplicate */
               istat = 1                  '/* status = 1 if operation failed */
            end if
         case BTRV.OPEN                              '/* open a Btrieve file */
            if istat = BTRV.NOFILE then     '/* valid Btrieve file not found */
               istat = 2                    '/* status = 2 if file not found */
            else                                    '/* not a file-not-found */
               istat = 1                  '/* status = 1 if operation failed */
            end if
         case else
      end select
   end if
   io.btrv = istat            '/* return operation status to calling program */
end function
